import { Injectable } from '@nestjs/common';
import {
  EventTopics,
  columnUniques,
  validateRelations,
} from 'src/core/strings/constants/interface.constants';
import { ItemEntity } from '../../entities/item.entity';
import { ItemModel } from '../../../data/models/item.model';
import { BaseCreateManager } from 'src/core/modules/domain/usecase/managers/base-create.manager';
import { ItemCreatedEvent } from '../../entities/event/item-created.event';

@Injectable()
export class CreateItemManager extends BaseCreateManager<ItemEntity> {
  async beforeProcess(): Promise<void> {
    Object.assign(this.data, {
      item_type: this.data.item_type.toLowerCase(),
    });

    if (this.data.limit_type) {
      Object.assign(this.data, {
        limit_type: this.data.limit_type.toLowerCase(),
      });
    }
    return;
  }

  async afterProcess(): Promise<void> {
    return;
  }

  get validateRelations(): validateRelations[] {
    return [];
  }

  get uniqueColumns(): columnUniques[] {
    return [{ column: 'name' }];
  }

  get eventTopics(): EventTopics[] {
    return [
      {
        topic: ItemCreatedEvent,
        data: this.data,
      },
    ];
  }

  get entityTarget(): any {
    return ItemModel;
  }
}
