import { Injectable } from '@nestjs/common';
import { CreateTimeGroupManager } from './managers/create-time-group.manager';
import { TimeGroupDataService } from '../../data/services/time-group-data.service';
import { TimeGroupEntity } from '../entities/time-group.entity';
import { DeleteTimeGroupManager } from './managers/delete-time-group.manager';
import { UpdateTimeGroupManager } from './managers/update-time-group.manager';
import { BaseDataTransactionOrchestrator } from 'src/core/modules/domain/usecase/orchestrators/base-data-transaction.orchestrator';
import { ActiveTimeGroupManager } from './managers/active-time-group.manager';
import { InactiveTimeGroupManager } from './managers/inactive-time-group.manager';
import { ConfirmTimeGroupManager } from './managers/confirm-time-group.manager';
import { STATUS } from 'src/core/strings/constants/base.constants';
import { BatchResult } from 'src/core/response/domain/ok-response.interface';
import { BatchConfirmTimeGroupManager } from './managers/batch-confirm-time-group.manager';
import { BatchInactiveTimeGroupManager } from './managers/batch-inactive-time-group.manager';
import { BatchActiveTimeGroupManager } from './managers/batch-active-time-group.manager';
import { BatchDeleteTimeGroupManager } from './managers/batch-delete-time-group.manager';
import { TABLE_NAME } from 'src/core/strings/constants/table.constants';

@Injectable()
export class TimeGroupDataOrchestrator extends BaseDataTransactionOrchestrator<TimeGroupEntity> {
  constructor(
    private createManager: CreateTimeGroupManager,
    private updateManager: UpdateTimeGroupManager,
    private deleteManager: DeleteTimeGroupManager,
    private activeManager: ActiveTimeGroupManager,
    private confirmManager: ConfirmTimeGroupManager,
    private inactiveManager: InactiveTimeGroupManager,
    private batchDeleteManager: BatchDeleteTimeGroupManager,
    private batchActiveManager: BatchActiveTimeGroupManager,
    private batchConfirmManager: BatchConfirmTimeGroupManager,
    private batchInactiveManager: BatchInactiveTimeGroupManager,
    private serviceData: TimeGroupDataService,
  ) {
    super();
  }

  async create(data): Promise<TimeGroupEntity> {
    this.createManager.setData(data);
    this.createManager.setService(this.serviceData, TABLE_NAME.TIME_GROUPS);
    await this.createManager.execute();
    await this.createManager.generateConfig();
    return this.createManager.getResult();
  }

  async update(dataId, data): Promise<TimeGroupEntity> {
    this.updateManager.setData(dataId, data);
    this.updateManager.setService(this.serviceData, TABLE_NAME.TIME_GROUPS);
    await this.updateManager.execute();
    return this.updateManager.getResult();
  }

  async delete(dataId): Promise<string> {
    this.deleteManager.setData(dataId);
    this.deleteManager.setService(this.serviceData, TABLE_NAME.TIME_GROUPS);
    await this.deleteManager.execute();
    return this.deleteManager.getResult();
  }

  async batchDelete(dataIds: string[]): Promise<BatchResult> {
    this.batchDeleteManager.setData(dataIds);
    this.batchDeleteManager.setService(
      this.serviceData,
      TABLE_NAME.TIME_GROUPS,
    );
    await this.batchDeleteManager.execute();
    return this.batchDeleteManager.getResult();
  }

  async active(dataId): Promise<string> {
    this.activeManager.setData(dataId, STATUS.ACTIVE);
    this.activeManager.setService(this.serviceData, TABLE_NAME.TIME_GROUPS);
    await this.activeManager.execute();
    return this.activeManager.getResult();
  }

  async batchActive(dataIds: string[]): Promise<BatchResult> {
    this.batchActiveManager.setData(dataIds, STATUS.ACTIVE);
    this.batchActiveManager.setService(
      this.serviceData,
      TABLE_NAME.TIME_GROUPS,
    );
    await this.batchActiveManager.execute();
    return this.batchActiveManager.getResult();
  }

  async confirm(dataId): Promise<string> {
    this.confirmManager.setData(dataId, STATUS.ACTIVE);
    this.confirmManager.setService(this.serviceData, TABLE_NAME.TIME_GROUPS);
    await this.confirmManager.execute();
    return this.confirmManager.getResult();
  }

  async batchConfirm(dataIds: string[]): Promise<BatchResult> {
    this.batchConfirmManager.setData(dataIds, STATUS.ACTIVE);
    this.batchConfirmManager.setService(
      this.serviceData,
      TABLE_NAME.TIME_GROUPS,
    );
    await this.batchConfirmManager.execute();
    return this.batchConfirmManager.getResult();
  }

  async inactive(dataId): Promise<string> {
    this.inactiveManager.setData(dataId, STATUS.INACTIVE);
    this.inactiveManager.setService(this.serviceData, TABLE_NAME.TIME_GROUPS);
    await this.inactiveManager.execute();
    return this.inactiveManager.getResult();
  }

  async batchInactive(dataIds: string[]): Promise<BatchResult> {
    this.batchInactiveManager.setData(dataIds, STATUS.INACTIVE);
    this.batchInactiveManager.setService(
      this.serviceData,
      TABLE_NAME.TIME_GROUPS,
    );
    await this.batchInactiveManager.execute();
    return this.batchInactiveManager.getResult();
  }
}
