import {
  HttpStatus,
  Injectable,
  UnprocessableEntityException,
} from '@nestjs/common';
import { BaseUpdateStatusManager } from 'src/core/modules/domain/usecase/managers/base-update-status.manager';
import { SeasonTypeEntity } from '../../entities/season-type.entity';
import {
  EventTopics,
  validateRelations,
} from 'src/core/strings/constants/interface.constants';
import { SeasonTypeModel } from '../../../data/models/season-type.model';
import { SeasonTypeChangeStatusEvent } from '../../entities/event/season-type-change-status.event';

@Injectable()
export class InactiveSeasonTypeManager extends BaseUpdateStatusManager<SeasonTypeEntity> {
  getResult(): string {
    return `Success inactive data ${this.result.name}`;
  }

  async validateProcess(): Promise<void> {
    const relationData = await this.dataServiceFirstOpt.getOneByOptions({
      where: {
        season_type_id: this.dataId,
      },
    });

    if (relationData) {
      throw new UnprocessableEntityException({
        statusCode: HttpStatus.UNPROCESSABLE_ENTITY,
        message: `Gagagl! data sudah berelasi dengen season period.`,
        error: 'Unprocessable Entity',
      });
    }

    return;
  }

  async beforeProcess(): Promise<void> {
    return;
  }

  async afterProcess(): Promise<void> {
    return;
  }

  get validateRelations(): validateRelations[] {
    return [];
  }

  get entityTarget(): any {
    return SeasonTypeModel;
  }

  get eventTopics(): EventTopics[] {
    return [
      {
        topic: SeasonTypeChangeStatusEvent,
        data: this.data,
      },
    ];
  }
}
