import { BaseBatchUpdateStatusManager } from 'src/core/modules/domain/usecase/managers/base-batch-update-status.manager';
import {
  EventTopics,
  validateRelations,
} from 'src/core/strings/constants/interface.constants';
import { BatchResult } from 'src/core/response/domain/ok-response.interface';
import {
  HttpStatus,
  Injectable,
  UnprocessableEntityException,
} from '@nestjs/common';
import { TransactionEntity } from 'src/modules/transaction/transaction/domain/entities/transaction.entity';
import { TransactionModel } from 'src/modules/transaction/transaction/data/models/transaction.model';
import { STATUS } from 'src/core/strings/constants/base.constants';
import { TransactionChangeStatusEvent } from 'src/modules/transaction/transaction/domain/entities/event/transaction-change-status.event';

@Injectable()
export class BatchCancelReconciliationManager extends BaseBatchUpdateStatusManager<TransactionEntity> {
  async validateData(data: TransactionEntity): Promise<void> {
    const transaction = await this.dataService.getOneByOptions({
      where: {
        id: data.id,
      },
    });

    if (
      ![STATUS.SETTLED, STATUS.WAITING].includes(transaction.status) &&
      !data.is_recap_transaction
    ) {
      throw new UnprocessableEntityException({
        statusCode: HttpStatus.UNPROCESSABLE_ENTITY,
        message: `Gagal! tidak bisa batalkan, karena status transaksi tidak settled atau waiting`,
        error: 'Unprocessable Entity',
      });
    }

    Object.assign(data, {
      reconciliation_mdr: this.data.reconciliation_mdr ?? null,
      reconciliation_confirm_by: this.user.name,
      reconciliation_confirm_date: new Date().getTime(),
      status: this.dataStatus,
      reconciliation_status: this.dataStatus,
      payment_date: this.data.payment_date,
    });

    // FIXME => VALIDATION GUARD CANCEL FOR RECONCILIATION FROM CASHIER
    if (data.is_recap_transaction) {
      // throw new UnprocessableEntityException({
      //   statusCode: HttpStatus.UNPROCESSABLE_ENTITY,
      //   message: `Gagagl! tidak dapat batalkan data rekap`,
      //   error: 'Unprocessable Entity',
      // });

      Object.assign(this.data, {
        reconciliation_confirm_by: null,
        reconciliation_confirm_date: null,
        reconciliation_status: STATUS.PENDING,
        payment_code: null,
        settlement_date: null,
      });
    }

    return;
  }

  beforeProcess(): Promise<void> {
    return;
  }

  afterProcess(): Promise<void> {
    return;
  }

  get validateRelations(): validateRelations[] {
    return [];
  }

  get entityTarget(): any {
    return TransactionModel;
  }

  get eventTopics(): EventTopics[] {
    return [
      {
        topic: TransactionChangeStatusEvent,
      },
    ];
  }

  getResult(): BatchResult {
    return this.result;
  }
}
