import { Injectable } from '@nestjs/common';
import { CreateUserPrivilegeManager } from './managers/create-user-privilege.manager';
import { UserPrivilegeDataService } from '../../../data/service/user-privilege-data.service';
import { UserPrivilegeEntity } from '../../entities/user-privilege.entity';
import { DeleteUserPrivilegeManager } from './managers/delete-user-privilege.manager';
import { UpdateUserPrivilegeManager } from './managers/update-user-privilege.manager';
import { BaseDataTransactionOrchestrator } from 'src/core/modules/domain/usecase/orchestrators/base-data-transaction.orchestrator';
import { ActiveUserPrivilegeManager } from './managers/active-user-privilege.manager';
import { InactiveUserPrivilegeManager } from './managers/inactive-user-privilege.manager';
import { ConfirmUserPrivilegeManager } from './managers/confirm-user-privilege.manager';
import { STATUS } from 'src/core/strings/constants/base.constants';
import { BatchResult } from 'src/core/response/domain/ok-response.interface';
import { BatchConfirmUserPrivilegeManager } from './managers/batch-confirm-user-privilege.manager';
import { BatchInactiveUserPrivilegeManager } from './managers/batch-inactive-user-privilege.manager';
import { BatchActiveUserPrivilegeManager } from './managers/batch-active-user-privilege.manager';
import { BatchDeleteUserPrivilegeManager } from './managers/batch-delete-user-privilege.manager';
import { TABLE_NAME } from 'src/core/strings/constants/table.constants';

@Injectable()
export class UserPrivilegeDataOrchestrator extends BaseDataTransactionOrchestrator<UserPrivilegeEntity> {
  constructor(
    private createManager: CreateUserPrivilegeManager,
    private updateManager: UpdateUserPrivilegeManager,
    private deleteManager: DeleteUserPrivilegeManager,
    private activeManager: ActiveUserPrivilegeManager,
    private confirmManager: ConfirmUserPrivilegeManager,
    private inactiveManager: InactiveUserPrivilegeManager,
    private batchDeleteManager: BatchDeleteUserPrivilegeManager,
    private batchActiveManager: BatchActiveUserPrivilegeManager,
    private batchConfirmManager: BatchConfirmUserPrivilegeManager,
    private batchInactiveManager: BatchInactiveUserPrivilegeManager,
    private serviceData: UserPrivilegeDataService,
  ) {
    super();
  }

  async create(data): Promise<UserPrivilegeEntity> {
    this.createManager.setData(data);
    this.createManager.setService(this.serviceData, TABLE_NAME.USER_PRIVILEGE);
    await this.createManager.execute();
    await this.createManager.generateConfig();
    return this.createManager.getResult();
  }

  async update(dataId, data): Promise<UserPrivilegeEntity> {
    this.updateManager.setData(dataId, data);
    this.updateManager.setService(this.serviceData, TABLE_NAME.USER_PRIVILEGE);
    await this.updateManager.execute();
    return this.updateManager.getResult();
  }

  async delete(dataId): Promise<string> {
    this.deleteManager.setData(dataId);
    this.deleteManager.setService(this.serviceData, TABLE_NAME.USER_PRIVILEGE);
    await this.deleteManager.execute();
    return this.deleteManager.getResult();
  }

  async batchDelete(dataIds: string[]): Promise<BatchResult> {
    this.batchDeleteManager.setData(dataIds);
    this.batchDeleteManager.setService(
      this.serviceData,
      TABLE_NAME.USER_PRIVILEGE,
    );
    await this.batchDeleteManager.execute();
    return this.batchDeleteManager.getResult();
  }

  async active(dataId): Promise<string> {
    this.activeManager.setData(dataId, STATUS.ACTIVE);
    this.activeManager.setService(this.serviceData, TABLE_NAME.USER_PRIVILEGE);
    await this.activeManager.execute();
    return this.activeManager.getResult();
  }

  async batchActive(dataIds: string[]): Promise<BatchResult> {
    this.batchActiveManager.setData(dataIds, STATUS.ACTIVE);
    this.batchActiveManager.setService(
      this.serviceData,
      TABLE_NAME.USER_PRIVILEGE,
    );
    await this.batchActiveManager.execute();
    return this.batchActiveManager.getResult();
  }

  async confirm(dataId): Promise<string> {
    this.confirmManager.setData(dataId, STATUS.ACTIVE);
    this.confirmManager.setService(this.serviceData, TABLE_NAME.USER_PRIVILEGE);
    await this.confirmManager.execute();
    return this.confirmManager.getResult();
  }

  async batchConfirm(dataIds: string[]): Promise<BatchResult> {
    this.batchConfirmManager.setData(dataIds, STATUS.ACTIVE);
    this.batchConfirmManager.setService(
      this.serviceData,
      TABLE_NAME.USER_PRIVILEGE,
    );
    await this.batchConfirmManager.execute();
    return this.batchConfirmManager.getResult();
  }

  async inactive(dataId): Promise<string> {
    this.inactiveManager.setData(dataId, STATUS.INACTIVE);
    this.inactiveManager.setService(
      this.serviceData,
      TABLE_NAME.USER_PRIVILEGE,
    );
    await this.inactiveManager.execute();
    return this.inactiveManager.getResult();
  }

  async batchInactive(dataIds: string[]): Promise<BatchResult> {
    this.batchInactiveManager.setData(dataIds, STATUS.INACTIVE);
    this.batchInactiveManager.setService(
      this.serviceData,
      TABLE_NAME.USER_PRIVILEGE,
    );
    await this.batchInactiveManager.execute();
    return this.batchInactiveManager.getResult();
  }
}
