import { Injectable } from '@nestjs/common';
import { CreateUserManager } from './managers/create-user.manager';
import { UserDataService } from '../../data/services/user-data.service';
import { UserEntity } from '../entities/user.entity';
import { DeleteUserManager } from './managers/delete-user.manager';
import { UpdateUserManager } from './managers/update-user.manager';
import { BaseDataTransactionOrchestrator } from 'src/core/modules/domain/usecase/orchestrators/base-data-transaction.orchestrator';
import { ActiveUserManager } from './managers/active-user.manager';
import { InactiveUserManager } from './managers/inactive-user.manager';
import { ConfirmUserManager } from './managers/confirm-user.manager';
import { STATUS } from 'src/core/strings/constants/base.constants';
import { BatchResult } from 'src/core/response/domain/ok-response.interface';
import { BatchConfirmUserManager } from './managers/batch-confirm-user.manager';
import { BatchInactiveUserManager } from './managers/batch-inactive-user.manager';
import { BatchActiveUserManager } from './managers/batch-active-user.manager';
import { BatchDeleteUserManager } from './managers/batch-delete-user.manager';
import { TABLE_NAME } from 'src/core/strings/constants/table.constants';
import { UpdatePasswordUserManager } from './managers/update-password-user.manager';

@Injectable()
export class UserDataOrchestrator extends BaseDataTransactionOrchestrator<UserEntity> {
  constructor(
    private createManager: CreateUserManager,
    private updateManager: UpdateUserManager,
    private deleteManager: DeleteUserManager,
    private activeManager: ActiveUserManager,
    private confirmManager: ConfirmUserManager,
    private inactiveManager: InactiveUserManager,
    private batchDeleteManager: BatchDeleteUserManager,
    private batchActiveManager: BatchActiveUserManager,
    private batchConfirmManager: BatchConfirmUserManager,
    private batchInactiveManager: BatchInactiveUserManager,
    private updatePasswordManager: UpdatePasswordUserManager,

    private serviceData: UserDataService,
  ) {
    super();
  }

  async create(data): Promise<UserEntity> {
    this.createManager.setData(data);
    this.createManager.setService(this.serviceData, TABLE_NAME.USER);
    await this.createManager.execute();
    await this.createManager.generateConfig();
    return this.createManager.getResult();
  }

  async update(dataId, data): Promise<UserEntity> {
    this.updateManager.setData(dataId, data);
    this.updateManager.setService(this.serviceData, TABLE_NAME.USER);
    await this.updateManager.execute();
    return this.updateManager.getResult();
  }

  async updatePassword(dataId, data): Promise<UserEntity> {
    this.updatePasswordManager.setData(dataId, data);
    this.updatePasswordManager.setService(this.serviceData, TABLE_NAME.USER);
    await this.updatePasswordManager.execute();
    return this.updatePasswordManager.getResult();
  }

  async delete(dataId): Promise<string> {
    this.deleteManager.setData(dataId);
    this.deleteManager.setService(this.serviceData, TABLE_NAME.USER);
    await this.deleteManager.execute();
    return this.deleteManager.getResult();
  }

  async batchDelete(dataIds: string[]): Promise<BatchResult> {
    this.batchDeleteManager.setData(dataIds);
    this.batchDeleteManager.setService(this.serviceData, TABLE_NAME.USER);
    await this.batchDeleteManager.execute();
    return this.batchDeleteManager.getResult();
  }

  async active(dataId): Promise<string> {
    this.activeManager.setData(dataId, STATUS.ACTIVE);
    this.activeManager.setService(this.serviceData, TABLE_NAME.USER);
    await this.activeManager.execute();
    return this.activeManager.getResult();
  }

  async batchActive(dataIds: string[]): Promise<BatchResult> {
    this.batchActiveManager.setData(dataIds, STATUS.ACTIVE);
    this.batchActiveManager.setService(this.serviceData, TABLE_NAME.USER);
    await this.batchActiveManager.execute();
    return this.batchActiveManager.getResult();
  }

  async confirm(dataId): Promise<string> {
    this.confirmManager.setData(dataId, STATUS.ACTIVE);
    this.confirmManager.setService(this.serviceData, TABLE_NAME.USER);
    await this.confirmManager.execute();
    return this.confirmManager.getResult();
  }

  async batchConfirm(dataIds: string[]): Promise<BatchResult> {
    this.batchConfirmManager.setData(dataIds, STATUS.ACTIVE);
    this.batchConfirmManager.setService(this.serviceData, TABLE_NAME.USER);
    await this.batchConfirmManager.execute();
    return this.batchConfirmManager.getResult();
  }

  async inactive(dataId): Promise<string> {
    this.inactiveManager.setData(dataId, STATUS.INACTIVE);
    this.inactiveManager.setService(this.serviceData, TABLE_NAME.USER);
    await this.inactiveManager.execute();
    return this.inactiveManager.getResult();
  }

  async batchInactive(dataIds: string[]): Promise<BatchResult> {
    this.batchInactiveManager.setData(dataIds, STATUS.INACTIVE);
    this.batchInactiveManager.setService(this.serviceData, TABLE_NAME.USER);
    await this.batchInactiveManager.execute();
    return this.batchInactiveManager.getResult();
  }
}
