import { Injectable } from '@nestjs/common';
import { CreateBannerManager } from './managers/create-banner.manager';
import { BannerDataService } from '../../data/services/banner-data.service';
import { BannerEntity } from '../entities/banner.entity';
import { DeleteBannerManager } from './managers/delete-banner.manager';
import { UpdateBannerManager } from './managers/update-banner.manager';
import { BaseDataTransactionOrchestrator } from 'src/core/modules/domain/usecase/orchestrators/base-data-transaction.orchestrator';
import { ActiveBannerManager } from './managers/active-banner.manager';
import { InactiveBannerManager } from './managers/inactive-banner.manager';
import { ConfirmBannerManager } from './managers/confirm-banner.manager';
import { STATUS } from 'src/core/strings/constants/base.constants';
import { BatchResult } from 'src/core/response/domain/ok-response.interface';
import { BatchConfirmBannerManager } from './managers/batch-confirm-banner.manager';
import { BatchInactiveBannerManager } from './managers/batch-inactive-banner.manager';
import { BatchActiveBannerManager } from './managers/batch-active-banner.manager';
import { BatchDeleteBannerManager } from './managers/batch-delete-banner.manager';
import { TABLE_NAME } from 'src/core/strings/constants/table.constants';

@Injectable()
export class BannerDataOrchestrator extends BaseDataTransactionOrchestrator<BannerEntity> {
  constructor(
    private createManager: CreateBannerManager,
    private updateManager: UpdateBannerManager,
    private deleteManager: DeleteBannerManager,
    private activeManager: ActiveBannerManager,
    private confirmManager: ConfirmBannerManager,
    private inactiveManager: InactiveBannerManager,
    private batchDeleteManager: BatchDeleteBannerManager,
    private batchActiveManager: BatchActiveBannerManager,
    private batchConfirmManager: BatchConfirmBannerManager,
    private batchInactiveManager: BatchInactiveBannerManager,
    private serviceData: BannerDataService,
  ) {
    super();
  }

  async create(data): Promise<BannerEntity> {
    this.createManager.setData(data);
    this.createManager.setService(this.serviceData, TABLE_NAME.BANNER);
    await this.createManager.execute();
    return this.createManager.getResult();
  }

  async update(dataId, data): Promise<BannerEntity> {
    this.updateManager.setData(dataId, data);
    this.updateManager.setService(this.serviceData, TABLE_NAME.BANNER);
    await this.updateManager.execute();
    return this.updateManager.getResult();
  }

  async delete(dataId): Promise<string> {
    this.deleteManager.setData(dataId);
    this.deleteManager.setService(this.serviceData, TABLE_NAME.BANNER);
    await this.deleteManager.execute();
    return this.deleteManager.getResult();
  }

  async batchDelete(dataIds: string[]): Promise<BatchResult> {
    this.batchDeleteManager.setData(dataIds);
    this.batchDeleteManager.setService(this.serviceData, TABLE_NAME.BANNER);
    await this.batchDeleteManager.execute();
    return this.batchDeleteManager.getResult();
  }

  async active(dataId): Promise<string> {
    this.activeManager.setData(dataId, STATUS.ACTIVE);
    this.activeManager.setService(this.serviceData, TABLE_NAME.BANNER);
    await this.activeManager.execute();
    return this.activeManager.getResult();
  }

  async batchActive(dataIds: string[]): Promise<BatchResult> {
    this.batchActiveManager.setData(dataIds, STATUS.ACTIVE);
    this.batchActiveManager.setService(this.serviceData, TABLE_NAME.BANNER);
    await this.batchActiveManager.execute();
    return this.batchActiveManager.getResult();
  }

  async confirm(dataId): Promise<string> {
    this.confirmManager.setData(dataId, STATUS.ACTIVE);
    this.confirmManager.setService(this.serviceData, TABLE_NAME.BANNER);
    await this.confirmManager.execute();
    return this.confirmManager.getResult();
  }

  async batchConfirm(dataIds: string[]): Promise<BatchResult> {
    this.batchConfirmManager.setData(dataIds, STATUS.ACTIVE);
    this.batchConfirmManager.setService(this.serviceData, TABLE_NAME.BANNER);
    await this.batchConfirmManager.execute();
    return this.batchConfirmManager.getResult();
  }

  async inactive(dataId): Promise<string> {
    this.inactiveManager.setData(dataId, STATUS.INACTIVE);
    this.inactiveManager.setService(this.serviceData, TABLE_NAME.BANNER);
    await this.inactiveManager.execute();
    return this.inactiveManager.getResult();
  }

  async batchInactive(dataIds: string[]): Promise<BatchResult> {
    this.batchInactiveManager.setData(dataIds, STATUS.INACTIVE);
    this.batchInactiveManager.setService(this.serviceData, TABLE_NAME.BANNER);
    await this.batchInactiveManager.execute();
    return this.batchInactiveManager.getResult();
  }
}
