import { Injectable } from '@nestjs/common';
import { CreateFaqManager } from './managers/create-faq.manager';
import { FaqDataService } from '../../data/services/faq-data.service';
import { FaqEntity } from '../entities/faq.entity';
import { DeleteFaqManager } from './managers/delete-faq.manager';
import { UpdateFaqManager } from './managers/update-faq.manager';
import { BaseDataTransactionOrchestrator } from 'src/core/modules/domain/usecase/orchestrators/base-data-transaction.orchestrator';
import { ActiveFaqManager } from './managers/active-faq.manager';
import { InactiveFaqManager } from './managers/inactive-faq.manager';
import { ConfirmFaqManager } from './managers/confirm-faq.manager';
import { STATUS } from 'src/core/strings/constants/base.constants';
import { BatchResult } from 'src/core/response/domain/ok-response.interface';
import { BatchConfirmFaqManager } from './managers/batch-confirm-faq.manager';
import { BatchInactiveFaqManager } from './managers/batch-inactive-faq.manager';
import { BatchActiveFaqManager } from './managers/batch-active-faq.manager';
import { BatchDeleteFaqManager } from './managers/batch-delete-faq.manager';
import { TABLE_NAME } from 'src/core/strings/constants/table.constants';
import { ChangePositionFaqManager } from './managers/change-position-faq.manager';

@Injectable()
export class FaqDataOrchestrator extends BaseDataTransactionOrchestrator<FaqEntity> {
  constructor(
    private createManager: CreateFaqManager,
    private updateManager: UpdateFaqManager,
    private deleteManager: DeleteFaqManager,
    private activeManager: ActiveFaqManager,
    private confirmManager: ConfirmFaqManager,
    private inactiveManager: InactiveFaqManager,
    private batchDeleteManager: BatchDeleteFaqManager,
    private batchActiveManager: BatchActiveFaqManager,
    private batchConfirmManager: BatchConfirmFaqManager,
    private batchInactiveManager: BatchInactiveFaqManager,
    private changePositionManager: ChangePositionFaqManager,
    private serviceData: FaqDataService,
  ) {
    super();
  }

  async changePostion(data): Promise<string> {
    this.changePositionManager.setData(data, 'sort_order');
    this.changePositionManager.setService(this.serviceData, TABLE_NAME.FAQ);
    await this.changePositionManager.execute();
    return this.changePositionManager.getResult();
  }

  async create(data): Promise<FaqEntity> {
    this.createManager.setData(data);
    this.createManager.setService(this.serviceData, TABLE_NAME.FAQ);
    await this.createManager.execute();
    return this.createManager.getResult();
  }

  async update(dataId, data): Promise<FaqEntity> {
    this.updateManager.setData(dataId, data);
    this.updateManager.setService(this.serviceData, TABLE_NAME.FAQ);
    await this.updateManager.execute();
    return this.updateManager.getResult();
  }

  async delete(dataId): Promise<string> {
    this.deleteManager.setData(dataId);
    this.deleteManager.setService(this.serviceData, TABLE_NAME.FAQ);
    await this.deleteManager.execute();
    return this.deleteManager.getResult();
  }

  async batchDelete(dataIds: string[]): Promise<BatchResult> {
    this.batchDeleteManager.setData(dataIds);
    this.batchDeleteManager.setService(this.serviceData, TABLE_NAME.FAQ);
    await this.batchDeleteManager.execute();
    return this.batchDeleteManager.getResult();
  }

  async active(dataId): Promise<string> {
    this.activeManager.setData(dataId, STATUS.ACTIVE);
    this.activeManager.setService(this.serviceData, TABLE_NAME.FAQ);
    await this.activeManager.execute();
    return this.activeManager.getResult();
  }

  async batchActive(dataIds: string[]): Promise<BatchResult> {
    this.batchActiveManager.setData(dataIds, STATUS.ACTIVE);
    this.batchActiveManager.setService(this.serviceData, TABLE_NAME.FAQ);
    await this.batchActiveManager.execute();
    return this.batchActiveManager.getResult();
  }

  async confirm(dataId): Promise<string> {
    this.confirmManager.setData(dataId, STATUS.ACTIVE);
    this.confirmManager.setService(this.serviceData, TABLE_NAME.FAQ);
    await this.confirmManager.execute();
    return this.confirmManager.getResult();
  }

  async batchConfirm(dataIds: string[]): Promise<BatchResult> {
    this.batchConfirmManager.setData(dataIds, STATUS.ACTIVE);
    this.batchConfirmManager.setService(this.serviceData, TABLE_NAME.FAQ);
    await this.batchConfirmManager.execute();
    return this.batchConfirmManager.getResult();
  }

  async inactive(dataId): Promise<string> {
    this.inactiveManager.setData(dataId, STATUS.INACTIVE);
    this.inactiveManager.setService(this.serviceData, TABLE_NAME.FAQ);
    await this.inactiveManager.execute();
    return this.inactiveManager.getResult();
  }

  async batchInactive(dataIds: string[]): Promise<BatchResult> {
    this.batchInactiveManager.setData(dataIds, STATUS.INACTIVE);
    this.batchInactiveManager.setService(this.serviceData, TABLE_NAME.FAQ);
    await this.batchInactiveManager.execute();
    return this.batchInactiveManager.getResult();
  }
}
