import { Injectable } from '@nestjs/common';
import { CreateGateManager } from './managers/create-gate.manager';
import { GateDataService } from '../../data/services/gate-data.service';
import { GateEntity } from '../entities/gate.entity';
import { DeleteGateManager } from './managers/delete-gate.manager';
import { UpdateGateManager } from './managers/update-gate.manager';
import { BaseDataTransactionOrchestrator } from 'src/core/modules/domain/usecase/orchestrators/base-data-transaction.orchestrator';
import { ActiveGateManager } from './managers/active-gate.manager';
import { InactiveGateManager } from './managers/inactive-gate.manager';
import { ConfirmGateManager } from './managers/confirm-gate.manager';
import { STATUS } from 'src/core/strings/constants/base.constants';
import { BatchResult } from 'src/core/response/domain/ok-response.interface';
import { BatchConfirmGateManager } from './managers/batch-confirm-gate.manager';
import { BatchInactiveGateManager } from './managers/batch-inactive-gate.manager';
import { BatchActiveGateManager } from './managers/batch-active-gate.manager';
import { BatchDeleteGateManager } from './managers/batch-delete-gate.manager';
import { TABLE_NAME } from 'src/core/strings/constants/table.constants';

@Injectable()
export class GateDataOrchestrator extends BaseDataTransactionOrchestrator<GateEntity> {
  constructor(
    private createManager: CreateGateManager,
    private updateManager: UpdateGateManager,
    private deleteManager: DeleteGateManager,
    private activeManager: ActiveGateManager,
    private confirmManager: ConfirmGateManager,
    private inactiveManager: InactiveGateManager,
    private batchDeleteManager: BatchDeleteGateManager,
    private batchActiveManager: BatchActiveGateManager,
    private batchConfirmManager: BatchConfirmGateManager,
    private batchInactiveManager: BatchInactiveGateManager,
    private serviceData: GateDataService,
  ) {
    super();
  }

  async create(data): Promise<GateEntity> {
    this.createManager.setData(data);
    this.createManager.setService(this.serviceData, TABLE_NAME.GATE);
    await this.createManager.execute();
    return this.createManager.getResult();
  }

  async update(dataId, data): Promise<GateEntity> {
    this.updateManager.setData(dataId, data);
    this.updateManager.setService(this.serviceData, TABLE_NAME.GATE);
    await this.updateManager.execute();
    return this.updateManager.getResult();
  }

  async delete(dataId): Promise<string> {
    this.deleteManager.setData(dataId);
    this.deleteManager.setService(this.serviceData, TABLE_NAME.GATE);
    await this.deleteManager.execute();
    return this.deleteManager.getResult();
  }

  async batchDelete(dataIds: string[]): Promise<BatchResult> {
    this.batchDeleteManager.setData(dataIds);
    this.batchDeleteManager.setService(this.serviceData, TABLE_NAME.GATE);
    await this.batchDeleteManager.execute();
    return this.batchDeleteManager.getResult();
  }

  async active(dataId): Promise<string> {
    this.activeManager.setData(dataId, STATUS.ACTIVE);
    this.activeManager.setService(this.serviceData, TABLE_NAME.GATE);
    await this.activeManager.execute();
    return this.activeManager.getResult();
  }

  async batchActive(dataIds: string[]): Promise<BatchResult> {
    this.batchActiveManager.setData(dataIds, STATUS.ACTIVE);
    this.batchActiveManager.setService(this.serviceData, TABLE_NAME.GATE);
    await this.batchActiveManager.execute();
    return this.batchActiveManager.getResult();
  }

  async confirm(dataId): Promise<string> {
    this.confirmManager.setData(dataId, STATUS.ACTIVE);
    this.confirmManager.setService(this.serviceData, TABLE_NAME.GATE);
    await this.confirmManager.execute();
    return this.confirmManager.getResult();
  }

  async batchConfirm(dataIds: string[]): Promise<BatchResult> {
    this.batchConfirmManager.setData(dataIds, STATUS.ACTIVE);
    this.batchConfirmManager.setService(this.serviceData, TABLE_NAME.GATE);
    await this.batchConfirmManager.execute();
    return this.batchConfirmManager.getResult();
  }

  async inactive(dataId): Promise<string> {
    this.inactiveManager.setData(dataId, STATUS.INACTIVE);
    this.inactiveManager.setService(this.serviceData, TABLE_NAME.GATE);
    await this.inactiveManager.execute();
    return this.inactiveManager.getResult();
  }

  async batchInactive(dataIds: string[]): Promise<BatchResult> {
    this.batchInactiveManager.setData(dataIds, STATUS.INACTIVE);
    this.batchInactiveManager.setService(this.serviceData, TABLE_NAME.GATE);
    await this.batchInactiveManager.execute();
    return this.batchInactiveManager.getResult();
  }
}
