import { Injectable } from '@nestjs/common';
import { CreateNewsManager } from './managers/create-news.manager';
import { NewsDataService } from '../../data/services/news-data.service';
import { NewsEntity } from '../entities/news.entity';
import { DeleteNewsManager } from './managers/delete-news.manager';
import { UpdateNewsManager } from './managers/update-news.manager';
import { BaseDataTransactionOrchestrator } from 'src/core/modules/domain/usecase/orchestrators/base-data-transaction.orchestrator';
import { ActiveNewsManager } from './managers/active-news.manager';
import { InactiveNewsManager } from './managers/inactive-news.manager';
import { ConfirmNewsManager } from './managers/confirm-news.manager';
import { STATUS } from 'src/core/strings/constants/base.constants';
import { BatchResult } from 'src/core/response/domain/ok-response.interface';
import { BatchConfirmNewsManager } from './managers/batch-confirm-news.manager';
import { BatchInactiveNewsManager } from './managers/batch-inactive-news.manager';
import { BatchActiveNewsManager } from './managers/batch-active-news.manager';
import { BatchDeleteNewsManager } from './managers/batch-delete-news.manager';
import { TABLE_NAME } from 'src/core/strings/constants/table.constants';

@Injectable()
export class NewsDataOrchestrator extends BaseDataTransactionOrchestrator<NewsEntity> {
  constructor(
    private createManager: CreateNewsManager,
    private updateManager: UpdateNewsManager,
    private deleteManager: DeleteNewsManager,
    private activeManager: ActiveNewsManager,
    private confirmManager: ConfirmNewsManager,
    private inactiveManager: InactiveNewsManager,
    private batchDeleteManager: BatchDeleteNewsManager,
    private batchActiveManager: BatchActiveNewsManager,
    private batchConfirmManager: BatchConfirmNewsManager,
    private batchInactiveManager: BatchInactiveNewsManager,
    private serviceData: NewsDataService,
  ) {
    super();
  }

  async create(data): Promise<NewsEntity> {
    this.createManager.setData(data);
    this.createManager.setService(this.serviceData, TABLE_NAME.NEWS);
    await this.createManager.execute();
    return this.createManager.getResult();
  }

  async update(dataId, data): Promise<NewsEntity> {
    this.updateManager.setData(dataId, data);
    this.updateManager.setService(this.serviceData, TABLE_NAME.NEWS);
    await this.updateManager.execute();
    return this.updateManager.getResult();
  }

  async delete(dataId): Promise<string> {
    this.deleteManager.setData(dataId);
    this.deleteManager.setService(this.serviceData, TABLE_NAME.NEWS);
    await this.deleteManager.execute();
    return this.deleteManager.getResult();
  }

  async batchDelete(dataIds: string[]): Promise<BatchResult> {
    this.batchDeleteManager.setData(dataIds);
    this.batchDeleteManager.setService(this.serviceData, TABLE_NAME.NEWS);
    await this.batchDeleteManager.execute();
    return this.batchDeleteManager.getResult();
  }

  async active(dataId): Promise<string> {
    this.activeManager.setData(dataId, STATUS.ACTIVE);
    this.activeManager.setService(this.serviceData, TABLE_NAME.NEWS);
    await this.activeManager.execute();
    return this.activeManager.getResult();
  }

  async batchActive(dataIds: string[]): Promise<BatchResult> {
    this.batchActiveManager.setData(dataIds, STATUS.ACTIVE);
    this.batchActiveManager.setService(this.serviceData, TABLE_NAME.NEWS);
    await this.batchActiveManager.execute();
    return this.batchActiveManager.getResult();
  }

  async confirm(dataId): Promise<string> {
    this.confirmManager.setData(dataId, STATUS.ACTIVE);
    this.confirmManager.setService(this.serviceData, TABLE_NAME.NEWS);
    await this.confirmManager.execute();
    return this.confirmManager.getResult();
  }

  async batchConfirm(dataIds: string[]): Promise<BatchResult> {
    this.batchConfirmManager.setData(dataIds, STATUS.ACTIVE);
    this.batchConfirmManager.setService(this.serviceData, TABLE_NAME.NEWS);
    await this.batchConfirmManager.execute();
    return this.batchConfirmManager.getResult();
  }

  async inactive(dataId): Promise<string> {
    this.inactiveManager.setData(dataId, STATUS.INACTIVE);
    this.inactiveManager.setService(this.serviceData, TABLE_NAME.NEWS);
    await this.inactiveManager.execute();
    return this.inactiveManager.getResult();
  }

  async batchInactive(dataIds: string[]): Promise<BatchResult> {
    this.batchInactiveManager.setData(dataIds, STATUS.INACTIVE);
    this.batchInactiveManager.setService(this.serviceData, TABLE_NAME.NEWS);
    await this.batchInactiveManager.execute();
    return this.batchInactiveManager.getResult();
  }
}
